from ....case import FixableCase, Category
import os
from infi.pyutils.lazy import cached_method, clear_cache
from logging import getLogger
from infi.execute import execute_assert_success
from pkg_resources import resource_filename


logger = getLogger(__name__)


ODM_FILEPATH = resource_filename(__name__, "infinidat.odm")
MSG_FILEPATH = resource_filename(__name__, "NFINIDAT.msg")
CLEAN_ODM_FILEPATH = resource_filename(__name__, "infinidat_clean_odm")
CAT_PATH = "/etc/methods/NFINIDAT.cat"

class OdmAttributes(FixableCase):
    @cached_method
    def _get_odm_attributes(self, odm_type):
        out = execute_assert_success(["odmget", odm_type]).get_stdout()
        return self._split_odm_attributes(out)

    def _split_odm_attributes(self, odm_content):
        import re
        result = re.findall("(\w+):\s+(.*?)(?:\n\n|$)", odm_content.strip(), re.DOTALL)
        def split_value(value):
            return re.findall("\S+ = \S+", value)
        result = [(key, split_value(value)) for key, value in result]
        return result

    def run_fix(self):
        # We run 'infinidat_clean_odm' in case there are *some* attributes already in the ODM
        execute_assert_success(["sh", CLEAN_ODM_FILEPATH])
        execute_assert_success(["odmadd", ODM_FILEPATH])
        execute_assert_success(["bosboot", "-a"])
        clear_cache(self)

    def run_test(self):
        with open(ODM_FILEPATH) as fd:
            odm_content = fd.read()
        infinidat_odm_attributes = self._split_odm_attributes(odm_content)
        for attr_key, attr_value in infinidat_odm_attributes:
            installed_attr_values = [value for key, value in self._get_odm_attributes(attr_key)]
            if attr_value not in installed_attr_values:
                self.fail("a {} value is missing in the ODM".format(attr_key))

    def get_description(self):
        return "Checking ODM attributes are installed"

    def get_kb_number(self):
        return "204014111"

class Catalog(FixableCase):
    def run_test(self):
        if not os.path.exists(CAT_PATH):
            self.fail("Catalog file does not exist")

        actual_content = execute_assert_success(["dspcat", "-g", CAT_PATH]).get_stdout()
        with open(MSG_FILEPATH, "rb") as fd:
           desired_content = fd.read()
        if actual_content != desired_content:
            self.fail("Catalog file is missing message entries")

    def run_fix(self):
        execute_assert_success(["gencat", CAT_PATH, MSG_FILEPATH])

    def get_description(self):
        return "Checking message catalog is installed"

    def get_kb_number(self):
        return "204161161"

def get_category():
    return Category("ODM", [OdmAttributes, Catalog])
